import requests
import os
from datetime import datetime

TARGET_FILE = 'targets.txt'
REPORT_PATH = 'report/output.html'

# 점검할 취약 룰 (단순 예시)
vuln_rules = {
    "Apache": "Apache 서버 정보 노출",
    "PHP": "PHP 사용 여부 노출",
    "nginx": "nginx 서버 정보 노출",
    "IIS": "IIS 서버 정보 노출",
}

results = []

def scan_target(url):
    try:
        res = requests.get(url, timeout=5)
        server_header = res.headers.get("Server", "알 수 없음")
        detected = []

        for key in vuln_rules:
            if key.lower() in server_header.lower():
                detected.append(vuln_rules[key])

        return {
            "url": url,
            "status": res.status_code,
            "server": server_header,
            "vulns": detected
        }
    except Exception as e:
        return {
            "url": url,
            "status": "연결 실패",
            "server": "N/A",
            "vulns": ["접속 불가 또는 오류"]
        }

def generate_report(results):
    total = len(results)
    vulnerable = sum(1 for r in results if r['vulns'])
    safe = total - vulnerable

    html = f"""<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>취약점 점검 리포트</title>
  <style>
    body {{ font-family: Arial; background: #f9f9f9; padding: 20px; }}
    table {{ border-collapse: collapse; width: 100%; background: #fff; }}
    th, td {{ border: 1px solid #ccc; padding: 8px; text-align: left; }}
    th {{ background: #eee; }}
    .vuln {{ color: red; font-weight: bold; }}
    .safe {{ color: green; }}
    h1 {{ color: #333; }}
  </style>
</head>
<body>
  <h1>취약점 점검 리포트</h1>
  <p>생성 일시: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}</p>
  <p>총 대상: {total}개 / 취약 대상: <span class="vuln">{vulnerable}</span> / 정상 대상: <span class="safe">{safe}</span></p>

  <table>
    <tr><th>URL</th><th>응답코드</th><th>서버 정보</th><th>취약점</th></tr>
"""

    for r in results:
        vuln_list = "<br>".join(r["vulns"]) if r["vulns"] else "<span class='safe'>없음</span>"
        html += f"<tr><td>{r['url']}</td><td>{r['status']}</td><td>{r['server']}</td><td>{vuln_list}</td></tr>\n"

    html += """
  </table>
</body>
</html>"""

    os.makedirs(os.path.dirname(REPORT_PATH), exist_ok=True)
    with open(REPORT_PATH, "w", encoding="utf-8") as f:
        f.write(html)

    print(f"[★] 리포트 생성 완료: {REPORT_PATH}")

# === 실행 ===
if __name__ == "__main__":
    if not os.path.exists(TARGET_FILE):
        print(f"[!] 대상 파일 {TARGET_FILE} 이 존재하지 않습니다.")
        exit(1)

    with open(TARGET_FILE, "r", encoding="utf-8") as f:
        urls = [line.strip() for line in f if line.strip()]

    print(f"[●] {len(urls)}개 URL 점검 시작...")
    for url in urls:
        print(f"[...] {url} 점검 중...")
        results.append(scan_target(url))

    generate_report(results)
